body{margin:0;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen,Ubuntu,Cantarell,Fira Sans,Droid Sans,Helvetica Neue,sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,Courier New,monospace}.App{text-align:center}.App-logo{height:40vmin;pointer-events:none}@media (prefers-reduced-motion: no-preference){.App-logo{animation:App-logo-spin infinite 20s linear}}.App-header{background-color:#282c34;min-height:100vh;display:flex;flex-direction:column;align-items:center;justify-content:center;font-size:calc(10px + 2vmin);color:#fff}.App-link{color:#61dafb}@keyframes App-logo-spin{0%{transform:rotate(0)}to{transform:rotate(360deg)}}.loader{position:absolute;top:calc(50% - 4em);left:calc(50% - 4em);width:6em;height:6em;border:1.1em solid rgba(0,0,0,.2);border-left:1.1em solid #000000;border-radius:50%;animation:load8 1.1s infinite linear;transition:opacity .3s}.loader--hide{opacity:0}@keyframes load8{0%{transform:rotate(0)}to{transform:rotate(360deg)}}.code-snippet{background:#eee;padding:1rem;margin-top:2rem;margin-left:2rem;margin-right:2rem;overflow:auto;border-radius:3px}.code-snippet .code{text-align:left;white-space:pre;word-spacing:normal;word-break:normal;word-wrap:normal;line-height:1.5}.circular--landscape{display:block;position:relative;width:200px;height:200px;overflow:hidden;border-radius:50%;margin-left:auto;margin-right:auto}.circular--landscape img{width:auto;height:100%;margin-left:-50px;max-width:initial}.circular--portrait{position:relative;width:200px;height:200px;overflow:hidden;border-radius:50%;margin-left:auto;margin-right:auto}.circular--portrait img{width:100%;height:auto}
